# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from django.core.urlresolvers import resolve, Resolver404
from django.template.loader import render_to_string
from django.utils.encoding import force_unicode, force_bytes

from .. import conf


class EnviBaseMiddleware(object):
    """ Base middleware class for environments. """

    def __init__(self):
        """ Attach the environment to self. """
        super(EnviBaseMiddleware, self).__init__()
        self.env = conf.ENVIRONMENT or {}

    def response_needs_updating(self, request, response):
        """ Determines whether the environment should be shown. """
        # Check for response types where the banner can't be inserted.
        # Adapted from django_debug_toolbar.
        _encoding = response.get('Content-Encoding', '')
        _type = response.get('Content-Type', '').split(';')[0]

        is_streaming = getattr(response, 'streaming', False)
        is_gzip = 'gzip' in _encoding
        is_invalid = _type not in ('text/html', 'application/xhtml+xml')

        if any([is_streaming, is_gzip, is_invalid]):
            return False

        try:
            is_admin = resolve(request.path).app_name == 'admin'
        except Resolver404:
            return False

        # Check that the current environment is enabled for the request.
        show_in_admin = self.env.get('SHOW_IN_ADMIN')
        show_in_site = self.env.get('SHOW_IN_SITE')
        return (show_in_admin and is_admin) or (show_in_site and not is_admin)

    def update_response(self, response):
        """ Abstract method. Needs to be implemented in subclasses. """
        raise NotImplemented

    def update_response_headers(self, response):
        """ Updates the response headers. """
        if response.get('Content-Length'):
            response['Content-Length'] = len(response.content)

    def process_response(self, request, response):
        """ Injects the banner prior to the response being returned. """
        if self.response_needs_updating(request=request, response=response):
            self.update_response(response=response)
            self.update_response_headers(response=response)

        return response


class EnviBaseTemplateMiddleware(EnviBaseMiddleware):
    """ Extends the EnviBaseMiddleware to support templates. """
    template_name = None

    def get_context_data(self, **kwargs):
        """ Allows update of context data. """
        context = self.env.get('CONTEXT', {})
        context.update(**kwargs)
        return context

    def update_response(self, response):
        # Extract the HTML content from the response.
        html = force_unicode(response.content)
        if html.find('</head>') < 0:
            # No closing </head>, assuming it's a HTML snippet.
            return response

        # Insert the style rule just before the closing </head> tag.
        head_html = render_to_string(
            template_name=self.template_name,
            context=self.get_context_data(),
        ) + '</head>'
        html = html.replace('</head>', head_html, 1)

        # Finally, update the response.
        response.content = force_bytes(html)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from django.conf import settings

from . import constants

ENVIRONMENT_KEY = getattr(
    settings,
    'ENVI_ENVIRONMENT_KEY',
    'local'
)

ENVIRONMENTS = getattr(
    settings,
    'ENVI_ENVIRONMENTS',
    constants.ENVIRONMENTS
)

ENVIRONMENT = getattr(
    settings,
    'ENVI_ENVIRONMENT',
    constants.ENVIRONMENTS[ENVIRONMENT_KEY]
)



# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

ENVIRONMENTS = {
    'local': {
        'CONTEXT': {
            'CONTENT': "ENVIRONMENT: LOCAL",
            'COLOR_A': "#a0a0a0",
            'COLOR_B': "#7c7c7c",
        },
        'SHOW_IN_ADMIN': True,
        'SHOW_IN_SITE': True,
    },
    'dev': {
        'CONTEXT': {
            'CONTENT': "ENVIRONMENT: DEVELOPMENT",
            'COLOR_A': "#2d88ff",
            'COLOR_B': "#1962c1",
        },
        'SHOW_IN_ADMIN': True,
        'SHOW_IN_SITE': True,
    },
    'stage': {
        'CONTEXT': {
            'CONTENT': "ENVIRONMENT: STAGING",
            'COLOR_A': "#ffba1e",
            'COLOR_B': "#dba11a",
        },
        'SHOW_IN_ADMIN': True,
        'SHOW_IN_SITE': True,
    },
    'production': {
        'CONTEXT': {
            'CONTENT': "ENVIRONMENT: PRODUCTION",
            'COLOR_A': "#ff1e1e",
            'COLOR_B': "#d61111",
        },
        'SHOW_IN_ADMIN': True,
        'SHOW_IN_SITE': False,
    }
}


# -*- coding: utf-8 -*-
from __future__ import absolute_import, unicode_literals

from .base import EnviBaseTemplateMiddleware


class EnviFooterMiddleware(EnviBaseTemplateMiddleware):
    """ Renders a banner-style template in to the response. """
    template_name = "envi/footer.html"



import os
from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), 'README.rst')) as readme:
    README = readme.read()

# allow setup.py to be run from any path
os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-envi',
    version='0.2',
    packages=find_packages(),
    include_package_data=True,
    license='BSD License',
    description='Lightweight set of middleware classes that inject visual ' \
                'indicators for each type of environment that a project is' \
                'deployed to.',
    long_description=README,
    url='http://github.com/teapow/django-envi',
    author='Thomas Power',
    author_email='thomaspwr@gmail.com',
    classifiers=[],
)
